package com.diagbot.aggregate;

import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushScaleDTO;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.MappingConfigFacade;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/8/19 13:50
 */
@Component
public class AssemblePushAggregate {

    @Autowired
    private MappingConfigFacade mappingConfigFacade;

    @DataProvider("assemblePushAll")
    public PushDTO assemblePushAll(
            @InvokeParameter("pushDTO") PushDTO data,
            @InvokeParameter("hospitalId") Long hospitalId,
            @DataConsumer("retLis") List<PushBaseDTO> retLis,
            @DataConsumer("retPacs") List<PushBaseDTO> retPacs,
            @DataConsumer("retDiagMap") Map<String, List<PushBaseDTO>> retDiagMap,
            @DataConsumer("retOperation") List<PushBaseDTO> retOperation,
            @DataConsumer("retDrug") List<PushBaseDTO> retDrug,
            @DataConsumer("retScale") List<PushScaleDTO> retScale,
            @DataConsumer("retNurse") List<PushBaseDTO> retNurse) {
        //检验
        if (ListUtil.isNotEmpty(retLis)) {
            data.setLis(retLis);
        }
        //检查
        if (ListUtil.isNotEmpty(retPacs)) {
            data.setPacs(retPacs);
        }
        //诊断
        if (MapUtils.isNotEmpty(retDiagMap)) {
            data.setDis(retDiagMap);
        }
        //手术
        if (ListUtil.isNotEmpty(retOperation)) {
            data.setOperations(retOperation);
        }
        //药品
        if (ListUtil.isNotEmpty(retDrug)) {
            data.setMedicines(retDrug);
        }
        //量表
        if (ListUtil.isNotEmpty(retScale)) {
            data.setScale(retScale);
        }
        //护理
        if (ListUtil.isNotEmpty(retNurse)) {
            data.setNurse(retNurse);
        }
        return data;
    }

    @DataProvider("retLis")
    public List<PushBaseDTO> retLis(@InvokeParameter("pushDTO") PushDTO data,
                                    @InvokeParameter("hospitalId") Long hospitalId) {
        //检验
        List<PushBaseDTO> retLis = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getLis())) {
            retLis = data.getLis();
            List<String> uniqueNameList = retLis.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.LisPack.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retLis.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retLis;
    }

    @DataProvider("retPacs")
    public List<PushBaseDTO> retPacs(@InvokeParameter("pushDTO") PushDTO data,
                                     @InvokeParameter("hospitalId") Long hospitalId) {
        //检查
        List<PushBaseDTO> retPacs = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getPacs())) {
            retPacs = data.getPacs();
            List<String> uniqueNameList = retPacs.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Pacs.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retPacs.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retPacs;
    }

    @DataProvider("retDiagMap")
    public Map<String, List<PushBaseDTO>> retDiagMap(@InvokeParameter("pushDTO") PushDTO data,
                                                     @InvokeParameter("hospitalId") Long hospitalId) {
        Map<String, List<PushBaseDTO>> retMap = new HashMap<>();
        if (data.getDis() != null && data.getDis().size() > 0) {
            retMap = data.getDis();
            List<String> uniqueNameList = Lists.newLinkedList();
            for (Map.Entry<String, List<PushBaseDTO>> entry : retMap.entrySet()) {
                if (ListUtil.isNotEmpty(entry.getValue())) {
                    uniqueNameList.addAll(entry.getValue().stream()
                            .map(i -> i.getName())
                            .collect(Collectors.toList()));
                }
            }
            if (ListUtil.isNotEmpty(uniqueNameList)) {
                Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                        = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Disease.getKey(), hospitalId, StatusEnum.Enable.getKey());

                for (Map.Entry<String, List<PushBaseDTO>> entry : retMap.entrySet()) {
                    if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                        entry.getValue().forEach(item -> {
                            if (uniqueNameMap.get(item.getName()) != null) {
                                item.setHisNameList(uniqueNameMap.get(item.getName())
                                        .values().stream()
                                        .map(i -> i.keySet())
                                        .flatMap(Collection::stream)
                                        .collect(Collectors.toList()));
                            }
                        });
                    }
                }
            }
        }
        return retMap;
    }

    @DataProvider("retOperation")
    public List<PushBaseDTO> retOperation(@InvokeParameter("pushDTO") PushDTO data,
                                          @InvokeParameter("hospitalId") Long hospitalId) {
        //手术
        List<PushBaseDTO> retOperation = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getOperations())) {
            retOperation = data.getOperations();
            List<String> uniqueNameList = retOperation.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Operation.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retOperation.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retOperation;
    }

    @DataProvider("retDrug")
    public List<PushBaseDTO> retDrug(@InvokeParameter("pushDTO") PushDTO data,
                                     @InvokeParameter("hospitalId") Long hospitalId) {
        //药品
        List<PushBaseDTO> retDrug = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getMedicines())) {
            retDrug = data.getMedicines();
            List<String> uniqueNameList = retDrug.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Drug.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retDrug.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retDrug;
    }

    @DataProvider("retScale")
    public List<PushScaleDTO> retScale(@InvokeParameter("pushDTO") PushDTO data,
                                       @InvokeParameter("hospitalId") Long hospitalId) {
        //量表
        List<PushScaleDTO> retScale = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getScale())) {
            retScale = data.getScale();
            List<String> uniqueNameList = retScale.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Scale.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retScale.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retScale;
    }

    @DataProvider("retNurse")
    public List<PushBaseDTO> retNurse(@InvokeParameter("pushDTO") PushDTO data,
                                      @InvokeParameter("hospitalId") Long hospitalId) {
        //量表
        List<PushBaseDTO> retNurse = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getNurse())) {
            retNurse = data.getNurse();
            List<String> uniqueNameList = retNurse.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Nurse.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retNurse.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retNurse;
    }
}