package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TranDeptLogDTO;
import com.diagbot.entity.TranLog;
import com.diagbot.facade.TranLogFacade;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.StatistByDeptVO;
import com.diagbot.vo.TranLogPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/10/25 11:23
 */
@RestController
@RequestMapping("/tran/log")
@Api(value = "对接日志记录相关API", tags = { "对接日志记录相关API" })
@SuppressWarnings("unchecked")
public class TranLogController {
    @Autowired
    private TranLogFacade tranLogFacade;

    @ApiOperation(value = "调用记录[by:zhaops]",
            notes = "patientId:病人标识<br>" +
                    "sceneName:服务名<br>" +
                    "deptName:部门名称"+
                    "startGmtReq:接收时间（起始区间）,格式【yyyy-MM-dd HH:mm:ss】<br>" +
                    "endGmtReq:接收时间（结束区间）,格式【yyyy-MM-dd HH:mm:ss】<br>" +
                    "startGmtResp:响应时间（起始区间）,格式【yyyy-MM-dd HH:mm:ss】<br>" +
                    "endGmtResp:响应时间（结束区间）,格式【yyyy-MM-dd HH:mm:ss】<br>" +
                    "successFlag:状态 1：成功，0：失败<br>" +
                    "subHospitalName:院区<br>")
    @PostMapping("/pageList")
    @SysLogger("pageList")
    public RespDTO<IPage<TranLog>> pageList(@Valid @RequestBody TranLogPageVO tranLogPageVO) {
        IPage<TranLog> data = tranLogFacade.pageList(tranLogPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "查看接口记录[by:zhaops]",
            notes = "id")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    @Transactional
    public RespDTO<TranLog> getRecordById(@RequestBody @Valid IdVO idVO) {
        TranLog data = tranLogFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据医院部门查看统计数据[by:wangfeng]",
            notes = " startGmtReq:接收时间（起始区间）,格式【yyyy-MM-dd HH:mm:ss】<br>" +
                    " endGmtReq:接收时间（结束区间）,格式【yyyy-MM-dd HH:mm:ss】<br>")
    @PostMapping("/getStatistByDept")
    @SysLogger("getStatistByDept")
    public RespDTO<List<TranDeptLogDTO>> getStatistByDept(@RequestBody @Valid StatistByDeptVO statistByDeptVO) {
        List<TranDeptLogDTO> data = tranLogFacade.getStatistByDeptName(statistByDeptVO);
        return RespDTO.onSuc(data);
    }




}
